# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

# coding=utf-8
import inspect

from webullsdkcore.request import ApiRequest

class PlaceOrderRequest(ApiRequest):
    def __init__(self):
        ApiRequest.__init__(self, "/trade/order/place", version='v1', method="POST", body_params={})
        self._stock_order = {}
        self.add_body_params("stock_order", self._stock_order)

    def add_stock_order_params(self, k, v):
        self._stock_order[k] = v

    def set_category(self, category):
        self.add_body_params("category", category)

    def set_account_id(self, account_id):
        self.add_body_params("account_id", account_id)

    def set_client_order_id(self, client_order_id):
        self.add_stock_order_params("client_order_id", client_order_id)

    def set_side(self, side):
        self.add_stock_order_params("side", side)

    def set_tif(self, tif):
        self.add_stock_order_params("tif", tif)

    def set_extended_hours_trading(self, extended_hours_trading):
        self.add_stock_order_params("extended_hours_trading", extended_hours_trading)

    def set_instrument_id(self, instrument_id):
        self.add_stock_order_params("instrument_id", instrument_id)

    def set_order_type(self, order_type):
        self.add_stock_order_params("order_type", order_type)

    def set_limit_price(self, limit_price):
        self.add_stock_order_params("limit_price", limit_price)

    def set_qty(self, quantity):
        self.add_stock_order_params("qty", quantity)

    def set_stop_price(self, stop_price):
        self.add_stock_order_params("stop_price", stop_price)

    def set_trailing_type(self, trailing_type):
        self.add_stock_order_params("trailing_type", trailing_type)

    def set_trailing_stop_step(self, trailing_stop_step):
        self.add_stock_order_params("trailing_stop_step", trailing_stop_step)

    def set_stock_order(self, client_order_id,  instrument_id, qty, side, tif, extended_hours_trading,
                        order_type, limit_price, stop_price, trailing_type,trailing_stop_step):
        self._stock_order.update({k: v for k, v in locals().items() if v is not None and k != 'self'})











# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

# coding=utf-8
import uuid
from webullsdktradehk.common.order_side import OrderSide
from webullsdktradehk.common.order_tif import OrderTIF
from webullsdktradehk.common.order_type import OrderType
from webullsdktradehk.request.cancel_order_request import CancelOrderRequest
from webullsdktradehk.request.get_open_orders_request import OpenOrdersListRequest
from webullsdktradehk.request.get_order_detail_request import OrderDetailRequest
from webullsdktradehk.request.get_today_orders_request import TodayOrdersListRequest
from webullsdktradehk.request.palce_order_request import PlaceOrderRequest
from webullsdktradehk.request.replace_order_request import ReplaceOrderRequest


class OrderOperation:
    def __init__(self, api_client):
        self.client = api_client

    def place_order(self, account_id, qty, instrument_id, side, client_order_id, order_type,
                    extended_hours_trading, tif, limit_price=None, stop_price=None,
                    trailing_type=None, trailing_stop_step=None):
        place_order_request = PlaceOrderRequest()
        place_order_request.set_account_id(account_id)
        place_order_request.set_stock_order(client_order_id=client_order_id, instrument_id=instrument_id, qty=qty,
                                            side=side, tif=tif, extended_hours_trading=extended_hours_trading,
                                            order_type=order_type, limit_price=limit_price, stop_price=stop_price,
                                            trailing_type=trailing_type, trailing_stop_step=trailing_stop_step)
        response = self.client.get_response(place_order_request)
        return response

    def replace_order(self, account_id, qty, instrument_id, side, client_order_id, order_type,
                      extended_hours_trading, tif, limit_price=None, stop_price=None,
                      trailing_type=None, trailing_stop_step=None):
        replace_order_request = ReplaceOrderRequest()
        replace_order_request.set_account_id(account_id)
        replace_order_request.set_stock_order(client_order_id=client_order_id, instrument_id=instrument_id, qty=qty,
                                              side=side, tif=tif, extended_hours_trading=extended_hours_trading,
                                              order_type=order_type, limit_price=limit_price, stop_price=stop_price,
                                              trailing_type=trailing_type, trailing_stop_step=trailing_stop_step)
        response = self.client.get_response(replace_order_request)
        return response

    def cancel_order(self, account_id, client_order_id):
        cancel_order_request = CancelOrderRequest()
        cancel_order_request.set_account_id(account_id)
        cancel_order_request.set_client_order_id(client_order_id)
        response = self.client.get_response(cancel_order_request)
        return response

    def list_today_orders(self, account_id, page_size=10, last_client_order_id=None):
        today_orders_list_request = TodayOrdersListRequest()
        today_orders_list_request.set_account_id(account_id)
        today_orders_list_request.set_page_size(page_size)
        if last_client_order_id is not None:
            today_orders_list_request.set_last_client_order_id(last_client_order_id)
        response = self.client.get_response(today_orders_list_request)
        return response

    def list_open_orders(self, account_id, page_size=10, last_client_order_id=None):
        open_orders_list_request = OpenOrdersListRequest()
        open_orders_list_request.set_account_id(account_id)
        open_orders_list_request.set_page_size(page_size)
        if last_client_order_id is not None:
            open_orders_list_request.set_last_client_order_id(last_client_order_id)
        response = self.client.get_response(open_orders_list_request)
        return response

    def query_order_detail(self, account_id, client_order_id):
        order_detail_request = OrderDetailRequest()
        order_detail_request.set_account_id(account_id)
        order_detail_request.set_client_order_id(client_order_id)
        response = self.client.get_response(order_detail_request)
        return response



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

# coding=utf-8

from json.tool import main
from webullsdktradehk.common.currency import Currency
from webullsdktradehk.request.get_account_balance_request import AccountBalanceRequest
from webullsdktradehk.request.get_account_positions_request import AccountPositionsRequest
from webullsdktradehk.request.get_account_profile_request import AccountProfileRequest
from webullsdktradehk.request.get_app_subscriptions import GetAppSubscriptions

class Account:
    def __init__(self, api_client):
        self.client = api_client

    def get_account_profile(self,  account_id):
        account_profile_request = AccountProfileRequest()
        account_profile_request.set_account_id(account_id)
        response = self.client.get_response(account_profile_request)
        return response

    def get_account_balance(self, account_id, total_asset_currency=Currency.HKD.name):
        account_balance_request = AccountBalanceRequest()
        account_balance_request.set_account_id(account_id)
        account_balance_request.set_total_asset_currency(total_asset_currency)
        response = self.client.get_response(account_balance_request)
        return response

    def get_account_position(self,  account_id,page_size=10, last_instrument_id=None):
        account_positions_request = AccountPositionsRequest()
        account_positions_request.set_account_id(account_id)
        account_positions_request.set_page_size(page_size)
        if last_instrument_id is not None:
            account_positions_request.set_last_instrument_id(last_instrument_id)
        response = self.client.get_response(account_positions_request)
        return response
    
    def get_app_subscriptions(self, subscription_id=None):
        app_subscriptions = GetAppSubscriptions()
        if subscription_id is not None:
            app_subscriptions.set_subscription_id(subscription_id)
        response = self.client.get_response(app_subscriptions)
        return response



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

# coding=utf-8
from webullsdkcore.request import ApiRequest


class AccountPositionsRequest(ApiRequest):
    def __init__(self):
        ApiRequest.__init__(self, "/account/positions", version='v1', method="GET", query_params={})

    def set_account_id(self, account_id):
        self.add_query_param("account_id", account_id)

    def set_page_size(self, page_size):
        self.add_query_param("page_size", page_size)

    def set_last_instrument_id(self, last_instrument_id):
        self.add_query_param("last_instrument_id", last_instrument_id)